from .fgcdrawer import FGCDrawer
from bitarray import bitarray
from .libs.hamming import *
from .libs.binarytools import *
from .libs.commonfunctions import CommonFunctions


class FGCCreator:
    """FGC Creator is used to create a Fancy Galaxy Code (FGC).
    """

    # FG Code constants
    VERSION_BITS = 4
    VERSION = 1

    @staticmethod
    def create_fgc(data, output_file, color_inner, color_outer, color_background, write_data_as_text) -> None:
        """Creates an fgc according to the given parameters."""
        all_data: bitarray = bitarray()

        CommonFunctions.print_seperation_line("=")
        print("Version:  %i" % FGCCreator.VERSION)
        print("Data:     %s" % data)

        # Convert version and data to bitarray
        version_bits = byte_to_bitarray(FGCCreator.VERSION, FGCCreator.VERSION_BITS)
        all_data.extend(version_bits)
        data_bits2 = ''.join(format(i, '08b') for i in bytearray(data, encoding='utf-8'))
        data_bits = bitarray(data_bits2)
        all_data.extend(data_bits)

        print("Binary:   ", end="")
        print_bitarray(all_data)

        # Add Hamming error correction bits
        str_data = all_data.to01()
        str_data = [int(bit) for bit in str_data]
        all_data_encoded = bitarray(hamming_code(str_data))

        print("Hamming:  ", end="")
        print_bitarray(all_data_encoded)

        str_data = all_data_encoded.to01()
        str_data = [int(bit) for bit in str_data]
        all_data_decoded = bitarray(hamming_decode(str_data))

        sanity_check = (all_data_decoded == all_data)
        print(f"Sanity:   {'PASSED' if sanity_check else 'FAILED! PLEASE CHECK INPUT AND HAMMING CODE.'}.")

        CommonFunctions.print_seperation_line("=")

        # Draw the actual simple vector graphic (svg)
        FGCDrawer.draw_fgc(
            data, 
            all_data_encoded, 
            output_file, 
            color_inner=color_inner, 
            color_outer=color_outer,
            color_background=color_background,
            write_data_as_text=write_data_as_text
        )



import hamming_codec

def encode_data(data):
    bytes_data = data.encode("utf-8")

    # split the bytes into chunks of 8 bits (1 byte)
    #chunks = [bytes_data[i:i+1] for i in range(0, len(bytes_data), 1)]
    chunks = [bytes_data[i:i+1] for i in range(0, len(bytes_data),1)]

    # encode each chunk separately
    encoded_chunks = []
    for chunk in chunks:
        int_data = int.from_bytes(chunk, byteorder='big')
        binary = hamming_codec.encode(int_data, 8)
        encoded_chunks.append(binary)

    encoded_message = ''.join(encoded_chunks)
    return encoded_message

def encode_version(version):
    binary = hamming_codec.encode(version, 4)
    encoded_message = ''.join(binary)
    return encoded_message

def decode_data(encoded_data):
    chunks = [encoded_data[i:i+12] for i in range(0, len(encoded_data), 12)]

    # Decode each chunk separately
    decoded_chunks = []
    for chunk in chunks:
        decoded_chunk = int(hamming_codec.decode(int(chunk, 2), 12), 2)
        decoded_chunks.append(decoded_chunk)

    # Convert the decoded chunks back to bytes
    decoded_bytes = bytes(decoded_chunks)

    # Decode the bytes to string
    decoded_message = decoded_bytes.decode("utf-8")
    return decoded_message

def decode_version(encoded_data):
    version = int(hamming_codec.decode(int(encoded_data, 2), 7), 2)
    return version

if __name__ == '__main__':
    data = "A croissant: 🥐"
    encoded_data = encode_data(data)
    decoded_data = decode_data(encoded_data)
    print("Original data:", data)
    print("Encoded data: ", encoded_data)
    print("Decoded data: ", decoded_data)



from ..fgc_tools.fgcdrawer import FGCDrawer
from bitarray import bitarray
from ..fgc_tools.libs.hamming_2 import *
from ..fgc_tools.libs.binarytools import *


class FGCCreator:
    """FGC Creator is used to create a Fancy Galaxy Code (FGC).
    """

    # FG Code constants
    VERSION_BITS = 4
    VERSION = 1

    @staticmethod
    def create_fgc(data, output_file, color_inner, color_outer, color_background, write_data_as_text) -> None:
        """Creates an fgc according to the given parameters."""

        print("=" * 80)
        print("Version: %i" % FGCCreator.VERSION)
        print("Data:    %s" % data)

        encoded_version = encode_version(FGCCreator.VERSION)
        encoded_data = encode_data(data)

        decoded_version = decode_version(encoded_version)
        decoded_data = decode_data(encoded_data)

        sanity_check = (decoded_data == data and decoded_version == FGCCreator.VERSION)
        print(f"Sanity:  {'PASSED' if sanity_check else 'FAILED! PLEASE CHECK INPUT AND HAMMING CODE.'}.")

        full_encoded_data = bitarray(encoded_version) + bitarray(encoded_data)

        # Draw the actual simple vector graphic (svg)
        FGCDrawer.draw_fgc(
            data, 
            full_encoded_data, 
            output_file, 
            color_inner=color_inner, 
            color_outer=color_outer,
            color_background=color_background,
            write_data_as_text=write_data_as_text
        )
